#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# ========== file: covid19data.py =============================================
"""
Created on Sun Oct 18 16:07:53 2020

@author: wor
"""
"""
Created on Fri Jun 19 15:19:29 2020

@author: davidworrall
version 0.01: Fri Jun 19 15:19:29 2020. Set up intitial cv read
version 0.02: 2020 31 Define class structure for a country
version 0.03: 2020 08 06 Add SMA
version 0.04: 2020 09 18 finish marked-line plotting routines * non-printing vals <- 0
version 0.05: 2020 09 19 finish x-axis 7day divisions and labels
version 0.06: 2020 09 23 regional sort according to classification in WorldRegionsUpdated.csv
            complete plot generation and write to file
version 0.07 begin investigation of auto anomoly detection. found TAD (twitter ...)
see https://blog.twitter.com/engineering/en_us/a/2015/introducing-practical-and-robust-anomaly-detection-in-a-time-series.html

version 0.08 code Split code into 3 files:
    - covid19data.py (this file)
    - covid19class.py    ... includes all of the above
    - covid19plotting.py ... includes all of the above

Covid19/Coronavirus global deaths
dataset:
Title:    COVID-19 Time Series Data
Author:   Shad Reynolds
URL:      https://data.world/shad/covid-19-time-series-data

based on deaths_global.csv, as reguarly updated
online comments:  @deviramanan2016 

@shad Do you have patient level covid19 data?
@jonathangill•last month

Jonathan Gill:              @ jonathangill•last month
Jonathangill021@gmail.com:  @ deeworr•last month

It would be very useful to have this - even if just basic data like sex and age.
-The US data is not geographically differentiated. I'm using https://github.com/nytimes/covid-19-data/blob/master/us-counties.csv . Does anyone have a better source?

- Hi Shad. Thanks for your work on this. One point and a question, if I may:
(a) The Country field for South Korea is in the form "Korea, South" which parses as two different values. Is it possible to change it to "South Korea"? makes the code less onerous.
(b) I notice there is a (weekly) "resonance" in most sequences. Do you know if this is a result of the collation methodologies? Dee.

Google Data: https://news.google.com/covid19/map?hl=en-US&gl=US&ceid=US%3Aen

The 2020 geographic population data is from the INTERNET USAGE STATISTICS website which also has land-area:
https://www.internetworldstats.com/stats.htm
https://www.internetworldstats.com/list3.htm

(list of regions at end of file)

Another source of data - check it out - is https://covidtracking.com/

"""
# ============================= This processes a single amalgamated covid input data (incl US states) file of all daily data

import os
import csv

import config                                                                   # the global variables initialization module

if config.reportProgress:                                                       
    from datetime import datetime
    productionTime = datetime.now()
    print ("=== Begin covid19data:\t", productionTime)

                                                                                # also should work with "us-counties20200516.csv"
                                                                                # NB OSofT is "deaths_global.csv"
colLabelRow         = 0                                                         # Some columnNames for the Deaths Data file to enhance readability

regionNameCol       = 0
countryNameCol      = 1
latCol_deathsData   = 2
longCol_deathsData  = 3
dateBeginCol        = 4

fname= config.allDeathsDataFilename                                             # full pathname for deaths_global2.csv

# =========== PULL THE JOHNS Hopkins DB  FILE =================================
with open(fname, newline='') as csvfile:
    ipdata = csv.reader(csvfile, delimiter=',', quotechar='"')
    deathsDB = []                                                               # initial list for initially storing the filedata 
    for row in ipdata:                                                          # bespoke province namefixes to eliminate commasin country banes 
                                                                                # to and standardise across DBs.   
        if row [countryNameCol] == "Korea, South": row [countryNameCol] = 'South Korea'
        if row [regionNameCol]   == "Bonaire, Sint Eustatius and Saba": row [regionNameCol] = 'Bonaire Sint Eustatius and Saba'
        if row [countryNameCol] == 'Taiwan*': row [regionNameCol] = 'Taiwan'
        if row [countryNameCol] == 'Congo (Kinshasa)': row [countryNameCol] = 'Congo-Kinshasa (DR)' ;row [regionNameCol] = ''
        if row [countryNameCol] == 'Congo (Brazzaville)': row [countryNameCol] = 'Congo-Brazzaville' ; row [regionNameCol] = ''
        if row [countryNameCol] == 'Holy See': row [countryNameCol] = 'Vatican'
        if row [countryNameCol] == 'Sint': row [countryNameCol] = 'Sint Maarten'
        if row [countryNameCol] == 'Pierre': row [countryNameCol] = 'Sant Pierre and Miquelon'
        if row [countryNameCol] == 'Terres': row [countryNameCol] = 'Terres Australes'
        if row [countryNameCol] == 'Smaller': row [countryNameCol] = 'Smaller Territories (BR)'
        if row [countryNameCol] == 'St Martin': row [countryNameCol] = 'Saint Martin'
        if row [countryNameCol] == "Cote d'Ivoire": row [countryNameCol] = 'Cote dIvoire'

        deathsDB += [[','.join(row)]]
for indx in range (1,len(deathsDB)):
    deathsDB[indx] = deathsDB[indx][0].split(',')                               # each row of the DB is indexed sequentially as per the original file
    try:                                                                        # convert death numwrical data to ints, geographicals to floats
        deathsDB[indx] [latCol_deathsData]    = float (deathsDB[indx] [latCol_deathsData])   # convert Latitude string to float
    except:
        deathsDB[indx] [latCol_deathsData]    = 0.0
    try:
        deathsDB[indx] [longCol_deathsData]   = float (deathsDB[indx] [longCol_deathsData])  # convert Longditude string to float
    except:
        deathsDB[indx] [longCol_deathsData]   = 0.0                             # `Long may not be defined
    for deathNr in range (longCol_deathsData+1, len (deathsDB[indx])):          # convert the rest (death numbers) to ints
        deathsDB[indx][deathNr] = int (deathsDB[indx][deathNr])

deathsDB[colLabelRow] = deathsDB[colLabelRow][0].split(',')                     # The column title texts
#           sunday      monday      tuesday   (wed = 1/22/20, the first date in the database, pre pad is used to startCharts on a Sun
datePad = ['1/19/20','1/20/20', '1/21/20']                                      # 1/22/20 (the first date in the DB) is a Wed, so pre pad for a Sunday start
dataPad = [0, 0, 0]                                                             # insert len ( datePad days of 0 data)
columnLabels = deathsDB[0]
dateSpacer = "."                                                                # a list of the column headings (dates etc)
#print ("============\nColumn Labels\n============\n", columnLabels)

config.datesList = datePad + columnLabels[4:]                                   # Sunday-padded dates   #print (dates)
#print (config.datesList)

deathsDB = deathsDB[1:]                                                         # drop the column headings

monthDict= {'01':'Jan ', '02':'Feb ', '03':'Mar ', '04':'Apr ','05':'May ', '06':'June',
            '07':'July', '08':'Aug ', '09':'Sep ', '10':'Oct ','11':'Nov ','12':'Dec ' }
def mmddyyyyy2yyyymmdd(spacer="" , alphaMonth = False):
    """ converts US dates in DB (mm/dd/yy) to international (yyyySmmSdd) in the datesList """
    for indx in range (len(config.datesList)):
        #print (config.datesList [indx] , end= " ")                             # dates start at col 4
        tmp = config.datesList[indx].split('/')                                 # [mm, dd, yy] as per the dataset
        year    = '20'+tmp[2]
        month   = ('0' +tmp[0])[-2:]
        day     = ('0' +tmp[1])[-2:]
        if alphaMonth:
            month = monthDict[month]
            config.datesList[indx]= year + month + day                          #[yyyyJan.dd]. S = spacer
        else:
            config.datesList[indx]= year + dateSpacer + month + day             #[yyyyJan.dd]. S = spacer


mmddyyyyy2yyyymmdd(".", alphaMonth=True )                                       # separate date components with a ","  
# print (config.datesList)

#deathsDB = deathsDB[1:]                                                        # drop the column headings
#print ("============\nColumn Labels\n============\n", config.datesList)

countriesList = []                                                              # collate all multiple-province country provinces into a single country entry
for province in deathsDB:                                                       # make a lit of all countries with provinces
    if province[0] != '':
        if province[1] not in countriesList:
            countriesList += [province[1]]

for country in countriesList:                                                   # compute the running sums of contries in provinces
    runningDeathsSum = ['',country,'','']+ [0] * (len (columnLabels) - 4)       # we just want the death data clugey
    for province in deathsDB:                                                   # find each province for that country
        if province [1] == country:
            for indx in range (4, len (runningDeathsSum)):
                runningDeathsSum [indx] = runningDeathsSum [indx] + province [indx]
#    print (runningDeathsSum)
    deathsDB += [runningDeathsSum]                                              # add new country summation to the deathsDB

# if no province, make province == country
for prov in deathsDB:
    if prov[regionNameCol]=="":
        prov[regionNameCol] = prov [countryNameCol]


for province in deathsDB:                                                        # Collates all provinces into a single dictionary entry
    if province[0] == "":
        config.provinceDeathsDict [province[1]]  = province                     # there is no province, the dict key is country
    else:
        config.provinceDeathsDict [province[0]]  = province                     # there is more than one province so use the dick key is the province
if 'Taiwan*' in config.provinceDeathsDict.keys():                               # kludge to get rid of the pesky bugger! 'Taiwan' also exits
    print ('config.provinceDeathsDict: "Taiwan" and "Taiwan*" exist in regionDeathsDict ...', end = '')
    try: 
        del config.provinceDeathsDict['Taiwan*']
        print ('"Taiwan*" deleted')
    except:
       print ('::::::::: WARNING: Could not delete "Taiwan*"')

def printAllProvinces():
    kees = sorted (config.provinceDeathsDict.keys())                            # alpha sort for pretty print
    for keeNr in range (len (kees)):
        print (kees[keeNr], end="\t\t")
#        if keeNr % 3 == 0: print()
